/**
 * WordPress Dependencies
 */
import { TextControl, SelectControl, BaseControl, Spinner, Draggable } from '@wordpress/components';
import { useState, useEffect, useMemo } from '@wordpress/element';
import { useSelect } from '@wordpress/data';
import { useEntityRecords, store as coreStore } from '@wordpress/core-data';

/**
 * External depedencies
 */
import { debounce } from 'lodash';

/**
 * Internal dependencies
 */
import { SortableList } from './sortable-list';

/**
 * The `PostSelector` component
 * 
 * A component for curating one or more posts
 */
export const PostSelector = (({ value, onChange }) => {

	// Create some states
	const [ postType, setPostType ] = useState( 'post' );
	const [ searchTerm, setSearchTerm ] = useState( '' );
	const [ page, setPage ] = useState( 1 );
	const [ loading, setLoading ] = useState( false );
	const [ showSearchResults, setShowSearchResults ] = useState( true );
	const [ inFocus, setInFocus ] = useState( false );
	const [ selectedPosts, setSelectedPosts ] = useState( [] );

	// Set initial values for post type and posts
	useEffect(() => {

		if (value?.postType) {
			setPostType(value.postType);
		}

		if (value?.posts && value.posts.length > 0) {
			setSelectedPosts(value.posts);
		}

	}, []);

	// Get the array of post type SelectControl options
	const getPostTypeOptions = () => {
		const postTypes = [
			{
				value: 'all',
				label: 'All'
			}
		];

		// Function to retrieve public post types
		const getPublicPostTypes = () => {
			const postTypes = useSelect(
				( select ) => select( coreStore ).getPostTypes( { per_page: -1 } ),
				[]
			);
			return useMemo( () => {
				const excludedPostTypes = [ 'attachment' ];
				return postTypes?.filter(
					( { viewable, slug } ) =>
						viewable && ! excludedPostTypes.includes( slug )
				);
			}, [ postTypes ] );
		};

		// Push public post types into SelectControl options array
		const publicPostTypes = getPublicPostTypes();
		if (publicPostTypes) {
			publicPostTypes.forEach((postType) => {
				postTypes.push({
					value: postType.slug,
					label: postType.name
				});
			});
		}

		return postTypes;
	}

	// Callback for when search result is selected
	const onSelectSearchResult = (post) => {

		// Deduplication
		if (selectedPosts.includes(post)) {
			return;
		}

		// Add the newly selected post
		selectedPosts.push(post);

		// Fire change event
		onChange({
			posts: selectedPosts,
			postType
		});

		// Set out of focus
		setInFocus(false);
	}

	// Callback for removing a selected post
	const onRemoveSelectedPost = (postToRemove) => {

		setSelectedPosts(selectedPosts.filter((post) => post !== postToRemove));

		// Fire change event
		onChange({
			posts: selectedPosts,
			postType
		});
	};

	// Callback for updating the sortable list
	const onSortableListUpdate = (val) => {

		// Pull out the post data from the SortableList objects
		const sortedPosts = val.map((post) => post.data);

		// Set the state
		setSelectedPosts(sortedPosts);

		// Fire change event
		onChange({
			posts: sortedPosts,
			postType
		});
	};

	// Create a SCHEDULED signpost
	const getSignpost = (signpost) => {
		return (
			<span
				className={`dmg-query-loop-block-enhancements-signpost dmg-query-loop-block-enhancements-signpost--${signpost}`}
			>
				{signpost.toUpperCase()}
			</span>
		);
	};

	// Define default query parameters
	let queryParams = {
		search: searchTerm,
		search_columns: ['post_title'],
		status: ['publish', 'future']
	};

	// If this is an integer, assume we are trying to find a post ID
	if (!isNaN(searchTerm)) {
		queryParams = {
			include: searchTerm
		};
	}

	// Query all post types
	if (postType === 'all') {
		queryParams.dmg_query_loop_block_enhancements_post_curator = 1;
	}

	// Define query parameters and fetch posts
	const selectorArgs = [
		'postType',
		(postType === 'all') ? 'post' : postType, // "all" does not cut it as a WP-JSON post type, so we need to use "post" while passing in our custom query param to get all post types
		queryParams,
	];

	const posts = useEntityRecords(...selectorArgs);

	// Set loading state
	if (loading === false && posts.isResolving) {
		setLoading(true);
	} else if (loading === true && posts.hasResolved) {
		setLoading(false);
	}

	// Generate display date
	const getDisplayDate = (post) => {

		// Calculate the timestamp difference
		const UTCDate = new Date().toUTCString();
		const nowTime = new Date(UTCDate).getTime();
		const then = new Date(`${post.date_gmt}Z`);
		const thenTime = then.getTime();

		const diff = Math.ceil((nowTime - thenTime) / 1000);

		// Generate the text
		let text = '';

		// For entries less than a day old, and not scheduled, use "time ago" language
		if (diff > 0 && diff < 86400) {
			if (diff < 60) {
				text = diff + ' seconds';
			} else if (diff < 3600) {
				const minutes = Math.ceil(diff / 60)
				text = minutes + ' minutes';
			} else {
				const hours = Math.ceil(diff / 60 / 60)
				text = hours + ' hours';
			}

			text += ' ago';
		} else {

			// Otherwise, output full date
			text = then.toLocaleDateString('en-us', { year:"numeric", month:"long", day:"numeric" } );
		}

		return text;
	}

	// Create markup for search results; filter out posts that are already selected
	const searchResults = (posts?.records) ? posts.records
		.filter((post) => !selectedPosts.map((selectedPost) => selectedPost.id).includes(post.id))
		.map((post) => {
			const editURL = `/wp-admin/post.php?post=${post.id}&action=edit`;
			return (
				<div
					className="dmg-query-loop-block-enhancements-post-selector__search-result"
					onClick={ () => onSelectSearchResult(post)}
				>
					<strong>
						{post?.status === 'future' &&
							getSignpost('scheduled')
						}
						{post.title.rendered}
					</strong>
					<date>{getDisplayDate(post)}</date>
					<a
						href={editURL}
						target="_blank"
						onClick={ (e) => e.stopPropagation() }
					>
						Edit
					</a>
					&nbsp;|&nbsp;
					<a
						href={post.link}
						target="_blank"
						onClick={ (e) => e.stopPropagation() }
					>
						View
					</a>
				</div>
			);
		}
	) : [];

	/**
	 * Debounced search function
	 */
	const search = debounce( (searchInput ) => {
		setSearchTerm( searchInput );
		setPage( 1 );
	}, 500 );

	return (
		<div className="dmg-query-loop-block-enhancements-post-selector">
			<BaseControl
				help="Configure a specific list of posts to display. Use the fields below select a post type and to search for posts by text or ID."
			/>
			<SelectControl
				options={ getPostTypeOptions() }
				value={ postType }
				label="Post type"
				onChange={ (selected) => setPostType(selected) }
			/>
			<div className="dmg-query-loop-block-enhancements-post-selector__search-field">
				<TextControl
					onChange={ search }
					placeholder="Enter a search term or post ID"
					onFocus={ () => setInFocus(true) }
				/>
				{inFocus && searchResults.length > 0 &&
					<div className="dmg-query-loop-block-enhancements-post-selector__search-results">
						{searchResults}
					</div>
				}
				{loading &&
					<Spinner />
				}
			</div>
			{selectedPosts.length > 0 &&
				<SortableList
					items={ selectedPosts.map((post) => {

						// Create a small component for display purposes
						const content = () => {
							return (
								<>
									<span
										className="dmg-query-loop-block-enhancements-post-selector__selected-post-title"
									>
										{post?.status === 'future' &&
											getSignpost('scheduled')
										}
										{post.title.rendered}
									</span>
									<span
										className="dmg-query-loop-block-enhancements-post-selector__selected-post-date"
									>
										{getDisplayDate(post)}
									</span>
								</>
							);
						}

						// Prepare an object for SortableList to accept
						return {
							data: post,
							content
						};
					})}
					onChange={ (val) => onSortableListUpdate(val) }
				/>
			}
		</div>
	);

});