/**
 * WordPress Dependencies
 */
import { addFilter } from '@wordpress/hooks';
import { useState, useEffect } from '@wordpress/element';
import { PanelBody, Button, Modal } from '@wordpress/components';
import { InspectorControls } from '@wordpress/block-editor';
import { useEntityRecords } from '@wordpress/core-data';
import { registerBlockVariation } from '@wordpress/blocks';

/**
 * Internal Dependencies
 */
import { PostSelector } from '../components/post-selector';
import { ReactComponent as Icon } from '../assets/post-curator.svg';

// Define the block to filter and the variation
const BLOCK_NAME = 'core/query';
const VARIATION_NAME = 'dmg/query-loop-block-variations/post-curator';

/**
 * The `PostCurator` component
 * 
 * A component for curating one or more posts for the Query core block
 */
export const PostCurator = (BlockEdit) => {

	return (props) => {

		// Check the namespace for filtering
		if (VARIATION_NAME !== props?.attributes?.namespace) {
			return <BlockEdit {...props} />;
		}

		const currentQuery = props.attributes.query;

		// Default arguments
		const selectorArgs = [
			'postType',
			currentQuery.postType,
			{
				include: currentQuery.include ?? '',
				orderBy: 'include',
				perPage: (currentQuery?.include) ? currentQuery?.include.length : 0,
				status: ['publish', 'future'],
				dmg_query_loop_block_enhancements_post_curator: 1
			}
		];

		// Fetch and sort the posts that have already been set
		const posts = useEntityRecords(...selectorArgs);

		console.log(posts);

		let sortedPosts = [];
		if (posts?.records?.length > 0) {
			const ids = currentQuery.include.split(',');
			ids.forEach((id) => {
				const post = posts.records.find((obj) => obj.id === Number.parseInt(id));
				if (post) {
					sortedPosts.push(post);
				}
			});
		}

		// When post selector is updated, change the values
		const onPostSelectorChange = (val) => {

			// Map each post to its ID
			const postIds = val.posts.map((post) => post.id);

			// Get the post type
			const postType = 'post';

			// Set the include and orderattributes
			props.setAttributes({
				query: {
					...currentQuery,
					postType,
					include: postIds.join(','),
					orderBy: 'include',
					perPage: postIds.length,
				},
			});
		}

		return (
			<>
				<InspectorControls>
					<PanelBody
						title="Curate Posts"
						className="dmg-query-loop-block-enhancements-post-curator"
					>
						<PostSelector
							onChange={ (val) => onPostSelectorChange(val) }
							value={ {
								posts: sortedPosts,
								postType: currentQuery.postType
							} }
						/>
					</PanelBody>
				</InspectorControls>
				<BlockEdit {...props} />
			</>
		);
	};
};

// Register the Post Curator plugin
addFilter(
	'editor.BlockEdit',
	BLOCK_NAME,
	PostCurator
);

// Register the block
registerBlockVariation(
	BLOCK_NAME,
	{
		name: VARIATION_NAME,
		title: 'Post Curator',
		description: 'Manually curate a selection of posts with a drag-and-drop interface.',
		attributes: {
			namespace: VARIATION_NAME
		},
		isActive: ['namespace'],
		icon: Icon,
		scope: ['inserter'],
		allowedControls: []
	}
);
